{% extends "beam/form.html" %}
{% load i18n %}
{% load beam_tags %}


{% block body_classes %}{{ block.super }} beam-update{% if component.model %}{% get_options component.model as options %} beam-update-{{ options.app_label }}-{{ options.model_name }}{% endif %}{% endblock %}


{% block title %}
    {% if heading %}
        {{ heading }}
    {% else %}
        {% get_options component.model as options %}
        {% blocktrans %}Update {{ object }}{% endblocktrans %}
        | {{ options.verbose_name|capfirst }}
    {% endif %}
    | {{ block.super }}
{% endblock %}


{% block heading %}
    {% if heading %}
        {{ heading }}
    {% else %}
        {% get_options component.model as options %}
        {% blocktrans %}Update {{ object }}{% endblocktrans %}
        <small class="text-muted">{{ options.verbose_name|capfirst }}</small>
    {% endif %}
{% endblock %}


{% block form_buttons_top %}
    <button type="submit" class="btn btn-outline-secondary" name="submit" value="save_and_continue_editing">{% trans "Save and continue editing" %}</button>
    <button type="submit" class="btn btn-primary" name="submit" value="save">{% trans "Save" %}</button>
{% endblock %}


{% block form_buttons %}
    <button type="submit" class="btn btn-outline-secondary" name="submit" value="save_and_continue_editing">{% trans "Save and continue editing" %}</button>
    <button type="submit" class="btn btn-primary" name="submit" value="save">{% trans "Save" %}</button>
{% endblock %}