{% extends "beam/base.html" %}
{% load i18n crispy_forms_tags %}

{% block title %}{{ title }} | {{ block.super }}{% endblock %}

{% block content %}
    <h1>{% block heading %}{{ title }}{% endblock %}</h1>
    {% if validlink %}
        <p>{% translate "Please enter your new password twice so we can verify you typed it in correctly." %}</p>
        <form method="post" >
            {% csrf_token %}
            {{ form|crispy }}
            <input type="submit" class="btn btn-primary" value="{% translate 'Change my password' %}" />
        </form>
    {% else %}
        <p class="alert alert-danger" role="alert">{% translate "The password reset link was invalid, possibly because it has already been used.  Please request a new password reset." %}</p>
        {% url "password_reset" as reset_url %}
        {% if reset_url %}
            <a href="{{ reset_url }}">{% trans "Forgotten your password or username?" %}</a>
        {% endif %}
    {% endif %}
{% endblock %}
