{% extends "beam/base.html" %}
{% load i18n %}

{% block title %}{{ title }} | {{ block.super }}{% endblock %}

{% block content %}
    <h1>{{ title }}</h1>
    <p>{% trans "You logged out successfully." %}</p>
    <p><a href="{% url "login" %}">{% trans "Log in again" %}</a></p>
{% endblock %}
