{% extends "beam/base.html" %}
{% load i18n crispy_forms_tags %}

{% block title %}{{ title }} | {{ block.super }}{% endblock %}

{% block content %}
    <h1>{% block heading %}{{ title }}{% endblock %}</h1>
    <p>{% translate 'Forgotten your password? Enter your email address below, and we’ll email instructions for setting a new one.' %}</p>

<form method="post">
    {% csrf_token %}
    {{ form|crispy }}
    <input type="submit" class="btn btn-primary" value="{% translate 'Reset my password' %}"/>
</form>
{% endblock %}
