{% load i18n %}


{% block messages_container %}
<section class="beam-alerts">

    {% block messages %}
    {% for message in messages %}

        {% block message_container %}
        <div class="alert alert-dismissible alert-{% if message.level_tag == 'error' %}danger{% else %}{{ message.level_tag }}{% endif %}" role="alert">

            {% block message %}
                {{ message }}
            {% endblock %}

            {% block buttons %}
                <button type="button" class="close" data-dismiss="alert" aria-label="{% trans 'Close' %}">
                    <span aria-hidden="true">&times;</span>
                </button>
            {% endblock %}

        </div>
        {% endblock %}

    {% endfor %}
    {% endblock %}

</section>
{% endblock %}