{% extends "beam/base.html" %}
{% load i18n %}

{% block title %}{{ title }} | {{ block.super }}{% endblock %}

{% block content %}
    <h1>{% block heading %}{{ title }}{% endblock %}</h1>
    <p>{% translate 'Your password was changed.' %}</p>
{% endblock %}
