{% extends "beam/base.html" %}
{% load i18n %}

{% block title %}{{ title }} | {{ block.super }}{% endblock %}

{% block content %}
    <h1>{% block heading %}{{ title }}{% endblock %}</h1>

    <p>{% translate 'We’ve emailed you instructions for setting your password, if an account exists with the email you entered. You should receive them shortly.' %}</p>

    <p>{% translate 'If you don’t receive an email, please make sure you’ve entered the address you registered with, and check your spam folder.' %}</p>
{% endblock %}
