{% extends "beam/base.html" %}
{% load i18n crispy_forms_tags %}

{% block title %}{{ title }} | {{ block.super }}{% endblock %}

{% block content %}
    <h1>{% block heading %}{{ title }}{% endblock %}</h1>

    {% if form.errors %}
        <p class="errornote">
            {% if form.errors.items|length == 1 %}{% translate "Please correct the error below." %}{% else %}{% translate "Please correct the errors below." %}{% endif %}
        </p>
    {% endif %}

    <p>{% translate 'Please enter your old password, for security’s sake, and then enter your new password twice so we can verify you typed it in correctly.' %}</p>

    <form method="post">
        {% csrf_token %}
        {{ form|crispy }}
        <input type="submit" class="btn btn-primary" value="{% translate 'Change my password' %}" />
    </form>
{% endblock %}
